/**
 *
 *  @file unit_test1.cpp
 *
 *  @brief cuThomasBatch kernel execution example.
 *
 *  cuThomasBatch is a software package provided by
 *  Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
 *
 *  @author Ivan Martinez-Perez ivan.martinez@bsc.es
 *  @author Pedro Valero-Lara   pedro.valero@bsc.es
 *
 **/
#include "ThomasMatrix.hpp"
#include <omp.h>
#include <iostream>

int N;
int BlockSize;

int main(int argc, char const *argv[])
{

	if(argc < 4 or argc > 4){
		std::cout << "Usage: ./run [system size] [#systems] [cuda block size]" << std::endl;
		return -1;
	}

	ThomasMatrix tm;
	int M = std::stoi(argv[1]);
	N = std::stoi(argv[2]);
	BlockSize  = std::stoi(argv[3]);
	
	//Loading a synthetic tridiagonal matrix into our structure
	tm = loadThomasMatrixSyn(M);
	
    allocateGlobalVariables(tm);
    initSimpleGlobalVariables(tm);
    allocateGPUVariables(tm);

 // Sequantial CPU Execution for correct error check
    execSeq(tm);
    initSimpleGlobalVariables(tm);

    initInterleavedGlobalVariables(tm);
	execCuThomas(tm);	
    freeGlobalVariables();
	
	return 0;

}


