/**
 *
 *  @file unit_test2.cpp
 *
 *  @brief cuThomasBatch kernel execution example with cuSparse and 
 *  cuSolve reference (gtsvStridedBatch and SpDcsrqrsvBatched) routines
 *
 *  cuThomasBatch is a software package provided by
 *  Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
 *
 *  @author Ivan Martinez-Perez ivan.martinez@bsc.es
 *  @author Pedro Valero-Lara   pedro.valero@bsc.es
 *
 **/

#include "ThomasMatrix.hpp"
#include <omp.h>
#include <iostream>

int N;
int BlockSize;

int main(int argc, char const *argv[])
{

	if(argc < 4 or argc > 4){
		std::cout << "Usage: ./run [system size] [#systems] [cuda block size]" << std::endl;
		return -1;
	}

	ThomasMatrix tm;
	int M = std::stoi(argv[1]);
	int row_size = M+1;
	int col_size = 3*(M-2)+4;
	N = std::stoi(argv[2]);
	BlockSize  = std::stoi(argv[3]);
	
	//Hardcode example
	/*int RowPtrA[row_size] = {1,3,6,9,12,col_size+1};
	int ColPtrA[col_size]={1,2,1,2,3,2,3,4,3,4,5,4,5};
	double Val_A[col_size]={3.0,2.0,1.0,3.0,2.0,1.0,3.0,2.0,1.0,3.0,2.0,1.0,3.0};
	double rhs[M] = {1.0,1.0,1.0,1.0,1.0};*/ 
	
	/*double *d = (double*) malloc(M*sizeof(double));
	double *u = (double*) malloc((M-1)*sizeof(double));
	double *l = (double*) malloc((M-1)*sizeof(double));

	tridiagonal_CSR_to_vector(RowPtrA,row_size,ColPtrA,col_size,
							  Val_A,&d[0],&u[0],&l[0],M);*/

	
	tm = loadThomasMatrixSyn(M);

    /*int i;
	printf("D = {");
	for(i=0;i<M;++i) printf("%f,",d[i]);
	printf("}\n");
	
	printf("U = {");
	for(i=0;i<M-1;++i) printf("%f,",u[i]);
	printf("}\n");

	printf("L = {");
	for(i=0;i<M-1;++i) printf("%f,",l[i]);
	printf("}\n");*/

	int *RowPtrB = (int*) malloc(row_size*sizeof(int));
	int *ColPtrB = (int*) malloc(col_size*sizeof(int));
	double *Val_B = (double*) malloc(col_size*sizeof(double));
	
	tridiagonal_vector_to_CSR(&RowPtrB[0],row_size,&ColPtrB[0],col_size,
							  &Val_B[0],tm.d,tm.a,tm.b,M);

	/*printf("ROW = {");
	for(i=0;i<row_size;++i) printf("%d,",RowPtrB[i]);
	printf("}\n");
	
	printf("COLUMN = {");
	for(i=0;i<col_size;++i) printf("%d,",ColPtrB[i]);
	printf("}\n");

	printf("VAL = {");
	for(i=0;i<col_size;++i) printf("%f,",Val_B[i]);
	printf("}\n");*/

	

    allocateGlobalVariables(tm);
    initSimpleGlobalVariables(tm);
    allocateGPUVariables(tm);

    // CPU Execution (seq + multi)
    execSeq(tm);
    initSimpleGlobalVariables(tm);
    execMulti(tm);

    // GPU Execution (cuThomas + cuSparse)
    initInterleavedGlobalVariables(tm);
	execCuThomas(tm);
 	execCuSparse(tm); 
	execCuSolverQR(M,N,RowPtrB,ColPtrB,Val_B,tm.rhs,tm);	
    freeGlobalVariables();
	
	return 0;

}


