/**
 *
 *  @file kernels.h
 *
 *  @brief cuThomasBatch kernel implementaion.
 *
 *  cuThomasBatch is a software package provided by
 *  Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
 *
 *  @author Ivan Martinez-Perez ivan.martinez@bsc.es
 *  @author Pedro Valero-Lara   pedro.valero@bsc.es
 *
 **/

extern int N;
extern int BlockSize;

struct ThomasMatrix {
    double *a;
    double *b; 
    double *d; 
    double *rhs;
    int batchCount;
    int M;
};

void allocateGlobalVariables(ThomasMatrix const& params);
void allocateGPUVariables(ThomasMatrix const& params);
void freeGlobalVariables();
void initSimpleGlobalVariables(ThomasMatrix const& params);
void initInterleavedGlobalVariables(ThomasMatrix const& params);
void execSeq(ThomasMatrix const& params);
void execMulti(ThomasMatrix const& params);
void execCuThomas(ThomasMatrix const& params);
void execCuSparse(ThomasMatrix const& params);
void execCuSolverQR(int SIZE, int BATCHCOUNT, int *CSRROW, 
					int *CSRCOLUMN, double *CSRVALUE, double *RHS,
					ThomasMatrix const& PARAMS);
